/*
The software in this package is distributed under the GNU General
Public License version 2 (with a special exception described below).

A copy of GNU General Public License (GPL) is included in this distribution,
in the file COPYING.GPL.

As a special exception, if other files instantiate templates or use macros
or inline functions from this file, or you compile this file and link it
with other works to produce a work based on this file, this file
does not by itself cause the resulting work to be covered
by the GNU General Public License.

However the source code for this file must still be made available
in accordance with section (3) of the GNU General Public License.

This exception does not invalidate any other reasons why a work based
on this file might be covered by the GNU General Public License.
*/
/** @file
 * @brief unit test for cron functions.
 *
 * @copyright Copyright &copy; 2009 by Intra2net AG
 *
 */

#define BOOST_TEST_DYN_LINK
#include <boost/test/unit_test.hpp>

#include <time.h>
#include <stdlib.h>
#include <stdexcept>

#include <cron.hpp>

using namespace std;
using namespace I2n::Time;

class TestCronPointFuncFixture
{
public:
    TestCronPointFuncFixture()
    {
        // our check-values are set for Germany
        setenv("TZ",":Europe/Berlin",1);
    }
};

BOOST_FIXTURE_TEST_SUITE(TestCronPointFunc, TestCronPointFuncFixture)

/////////////////////////////////////////////////////
// Points in time
/////////////////////////////////////////////////////

BOOST_AUTO_TEST_CASE(NotYetToday)
{
    WeekCron cron("2345",3600);
    BOOST_CHECK_EQUAL( static_cast<time_t>(1233100800), cron.get_next_run(1233099657));
}

BOOST_AUTO_TEST_CASE(Now)
{
    WeekCron cron("2345",3600);
    BOOST_CHECK_EQUAL( static_cast<time_t>(1233187200), cron.get_next_run(1233100800));
}

BOOST_AUTO_TEST_CASE(LaterToday)
{
    WeekCron cron("2345",3600);
    BOOST_CHECK_EQUAL( static_cast<time_t>(1233187200), cron.get_next_run(1233100801));
}

BOOST_AUTO_TEST_CASE(Tomorrow)
{
    WeekCron cron("45",3600);
    BOOST_CHECK_EQUAL( static_cast<time_t>(1233187200), cron.get_next_run(1233099657));
}

BOOST_AUTO_TEST_CASE(NextWeek)
{
    WeekCron cron("1",3600);
    BOOST_CHECK_EQUAL( static_cast<time_t>(1233532800), cron.get_next_run(1233099657));
}

BOOST_AUTO_TEST_CASE(NextWeekFromToday)
{
    WeekCron cron("13",3600);
    BOOST_CHECK_EQUAL( static_cast<time_t>(1233532800), cron.get_next_run(1233100801));
}

BOOST_AUTO_TEST_CASE(StartMidnight)
{
    WeekCron cron("2345",0);
    BOOST_CHECK_EQUAL( static_cast<time_t>(1233097200), cron.get_next_run(1233097199));
}

BOOST_AUTO_TEST_CASE(StartMidnightTomorrow)
{
    WeekCron cron("2345",0);
    BOOST_CHECK_EQUAL( static_cast<time_t>(1233183600), cron.get_next_run(1233097200));
}

BOOST_AUTO_TEST_CASE(StartLastDaysec)
{
    WeekCron cron("2345",86399);
    BOOST_CHECK_EQUAL( static_cast<time_t>(1233183599), cron.get_next_run(1233097200));
}

BOOST_AUTO_TEST_CASE(HalfOpenInterval)
{
    WeekCron cron("2345",86400);
    BOOST_REQUIRE_THROW(cron.get_next_run(1233097200),std::runtime_error);
}

BOOST_AUTO_TEST_CASE(LongBeforeDSTBackwards)
{
    WeekCron cron("0123456",75600);
    // calc at: Sun Oct 26 00:00:00 CEST 2008
    // expected next run: Sun Oct 26 21:00:00 CET 2008
    BOOST_CHECK_EQUAL( static_cast<time_t>(1225051200), cron.get_next_run(1224972000));
}

BOOST_AUTO_TEST_CASE(BeforeDSTBackwards1)
{
    WeekCron cron("0123456",7200);
    // calc at: Sun Oct 26 01:59:59 CEST 2008
    // expected next run: Sun Oct 26 02:00:00 CET 2008
    BOOST_CHECK_EQUAL( static_cast<time_t>(1224982800), cron.get_next_run(1224979199));
}

BOOST_AUTO_TEST_CASE(BeforeDSTBackwards2)
{
    WeekCron cron("0123456",7200);
    // calc at: Sun Oct 26 02:59:59 CEST 2008
    // expected next run: Sun Oct 26 02:00:00 CET 2008
    BOOST_CHECK_EQUAL( static_cast<time_t>(1224982800), cron.get_next_run(1224982799));
}

BOOST_AUTO_TEST_CASE(AtDSTBackwards)
{
    WeekCron cron("0123456",7200);
    // calc at: Sun Oct 26 02:00:00 CET 2008
    // expected next run: Mon Oct 27 02:00:00 CET 2008
    BOOST_CHECK_EQUAL( static_cast<time_t>(1225069200), cron.get_next_run(1224982800));
}

BOOST_AUTO_TEST_CASE(DuringDSTBackwards)
{
    WeekCron cron("0123456",7200);
    // calc at: Sun Oct 26 02:00:01 CET 2008
    // expected next run: Mon Oct 27 02:00:00 CET 2008
    BOOST_CHECK_EQUAL( static_cast<time_t>(1225069200), cron.get_next_run(1224982801));
}

BOOST_AUTO_TEST_CASE(EndDSTBackwards1)
{
    WeekCron cron("0123456",10800);
    // calc at: Sun Oct 26 02:00:00 CEST 2008
    // expected next run: Sun Oct 26 03:00:00 CET 2008
    BOOST_CHECK_EQUAL( static_cast<time_t>(1224986400), cron.get_next_run(1224979200));
}

BOOST_AUTO_TEST_CASE(EndDSTBackwards2)
{
    WeekCron cron("0123456",10800);
    // calc at: Sun Oct 26 02:00:01 CEST 2008
    // expected next run: Sun Oct 26 03:00:00 CET 2008
    BOOST_CHECK_EQUAL( static_cast<time_t>(1224986400), cron.get_next_run(1224979201));
}

BOOST_AUTO_TEST_CASE(EndDSTBackwards3)
{
    WeekCron cron("0123456",10799);
    // calc at: Sun Oct 26 02:00:01 CEST 2008
    // expected next run: Sun Oct 26 02:59:59 CET 2008
    BOOST_CHECK_EQUAL( static_cast<time_t>(1224986399), cron.get_next_run(1224979201));
}

BOOST_AUTO_TEST_CASE(EndDSTBackwards4)
{
    WeekCron cron("0123456",10799);
    // calc at: Sun Oct 26 02:03:20 CET 2008
    // expected next run: Sun Oct 26 02:59:59 CET 2008
    BOOST_CHECK_EQUAL( static_cast<time_t>(1224986399), cron.get_next_run(1224983000));
}

BOOST_AUTO_TEST_CASE(OverDSTBackwards)
{
    WeekCron cron("234",10800);
    // calc at: Sat Oct 25 05:00:00 CEST 2008
    // expected next run: Tue Oct 28 03:00:00 CET 2008
    BOOST_CHECK_EQUAL( static_cast<time_t>(1225159200), cron.get_next_run(1224903600));
}

BOOST_AUTO_TEST_CASE(OverDSTBackwardsDaychange)
{
    WeekCron cron("234",0);
    // calc at: Sat Oct 25 00:00:00 CEST 2008
    // expected next run: Tue Oct 28 00:00:00 CET 2008
    BOOST_CHECK_EQUAL( static_cast<time_t>(1225148400), cron.get_next_run(1224885600));
}

BOOST_AUTO_TEST_CASE(LongBeforeDSTForward)
{
    WeekCron cron("0123456",75600);
    // calc at: Sat Mar 28 22:00:00 CET 2009
    // expected next run: Sun Mar 29 21:00:00 CEST 2009
    BOOST_CHECK_EQUAL( static_cast<time_t>(1238353200), cron.get_next_run(1238274000));
}

BOOST_AUTO_TEST_CASE(BeforeDSTForward1)
{
    WeekCron cron("0123456",7200);
    // calc at: Sun Mar 29 01:59:59 CET 2009
    // expected next run: Sun Mar 29 03:00:00 CEST 2009
    // glibc update: BOOST_CHECK_EQUAL( static_cast<time_t>(1238288400), cron.get_next_run(1238288399));
    BOOST_CHECK_EQUAL( static_cast<time_t>(1238288400), cron.get_next_run(1238288399));
}

BOOST_AUTO_TEST_CASE(BeforeDSTForward2)
{
    WeekCron cron("0123456",0);
    // calc at: Sun Mar 29 01:59:59 CET 2009
    // expected next run: Mon Mar 30 00:00:00 CEST 2009
    BOOST_CHECK_EQUAL( static_cast<time_t>(1238364000), cron.get_next_run(1238288399));
}

BOOST_AUTO_TEST_CASE(AtDSTForward)
{
    WeekCron cron("0123456",7200);
    // calc at: Sun Mar 29 03:00:00 CEST 2009
    // expected next run: Mon Mar 30 02:00:00 2009
    BOOST_CHECK_EQUAL( static_cast<time_t>(1238371200), cron.get_next_run(1238288400));
}

BOOST_AUTO_TEST_CASE(DuringDSTForward)
{
    WeekCron cron("0123456",9000);
    // calc at: Sun Mar 29 01:59:59 CET 2009
    // expected next run: Sun Mar 29 03:30:00 CEST 2009
    // glibc update: BOOST_CHECK_EQUAL( static_cast<time_t>(1238290200), cron.get_next_run(1238288399));
    BOOST_CHECK_EQUAL( static_cast<time_t>(1238290200), cron.get_next_run(1238288399));
}

BOOST_AUTO_TEST_CASE(EndDSTForward1)
{
    WeekCron cron("0123456",10800);
    // calc at: Sun Mar 29 01:59:59 CET 2009
    // expected next run: Sun Mar 29 03:00:00 CEST 2009
    BOOST_CHECK_EQUAL( static_cast<time_t>(1238288400), cron.get_next_run(1238288399));
}

BOOST_AUTO_TEST_CASE(EndDSTForward2)
{
    WeekCron cron("0123456",10800);
    // calc at: Sun Mar 29 03:00:00 CEST 2009
    // expected next run: Mon Mar 30 03:00:00 CEST 2009
    BOOST_CHECK_EQUAL( static_cast<time_t>(1238374800), cron.get_next_run(1238288400));
}

BOOST_AUTO_TEST_CASE(EndDSTForward3)
{
    WeekCron cron("0123456",10800);
    // calc at: Sun Mar 29 04:00:00 CEST 2009
    // expected next run: Mon Mar 30 03:00:00 CEST 2009
    BOOST_CHECK_EQUAL( static_cast<time_t>(1238374800), cron.get_next_run(1238292000));
}

BOOST_AUTO_TEST_CASE(EndDSTForward4)
{
    WeekCron cron("0123456",86339);
    // calc at: Sun Mar 29 11:42:06 CEST 2009
    // expected next run: Sun Mar 29 23:58:59 CEST 2009
    BOOST_CHECK_EQUAL( static_cast<time_t>(1238363939), cron.get_next_run(1238319726));
}

BOOST_AUTO_TEST_CASE(OverDSTForward)
{
    WeekCron cron("234",10800);
    // calc at: Sat Mar 28 23:00:00 CET 2009
    // expected next run: Tue Mar 31 03:00:00 CEST 2009
    BOOST_CHECK_EQUAL( static_cast<time_t>(1238461200), cron.get_next_run(1238277600));
}

BOOST_AUTO_TEST_CASE(OverDSTForwardDaychange)
{
    WeekCron cron("1234",0);
    // calc at: Sun Mar 29 00:00:00 CET 2009
    // expected next run: Mon Mar 30 00:00:00 CEST 2009
    BOOST_CHECK_EQUAL( static_cast<time_t>(1238364000), cron.get_next_run(1238281200));
}

BOOST_AUTO_TEST_CASE(StartHourStaysTheSameTill2038)
{
    int daysec = 79200;

    // Schedule daily at 22h from 1970 till 01.01.2038. Check every 90 minutes.
    WeekCron cron("0123456", daysec);
    for (time_t now = 86400*15; now < 2145916800; now += 90*60)
    {
        time_t result = cron.get_next_run(now);

        // Calculate unix time for the begin of the day
        struct tm calc_daybegin;
        bool conversion_ok = (localtime_r(&result, &calc_daybegin) != NULL);
        BOOST_CHECK_EQUAL(true, conversion_ok);

        int remain=daysec;

        calc_daybegin.tm_hour=remain/3600;
        remain-=calc_daybegin.tm_hour*3600;

        calc_daybegin.tm_min=remain/60;
        remain-=calc_daybegin.tm_min*60;

        calc_daybegin.tm_sec=remain;

        // tm_isdst=-1 means to use the dst in use at the given time
        calc_daybegin.tm_isdst=-1;

        time_t daytime = mktime(&calc_daybegin);

        BOOST_CHECK_EQUAL(daytime, result);
    }
}

BOOST_AUTO_TEST_SUITE_END()
